function [decisions,nrpairs,weights] = read_decisions(methods,prefix,uniquenessthreshold,maxpairs,maxrep)
% function [decisions,nrpairs,weights] = read_decisions(methods,prefix,uniquenessthreshold,maxpairs,maxrep)
%
% Copyright (c) 2008-2014  Joris Mooij  <j.m.mooij@uva.nl>
% All rights reserved.  See the file LICENSE for license terms.

  nrmethods = length(methods);

  % load metadata
  global WEBDAVDIR;
  webdavdir = WEBDAVDIR;
  metadata = load(sprintf('%s/pairmeta.txt',webdavdir));
  nrpairs = size(metadata,1);
  if nrpairs > maxpairs
    nrpairs = maxpairs;
  end
  for pair=1:nrpairs
    if metadata(pair,1) ~= pair 
      error('Assumption on metadata not satisfied');
    end
  end

  % set weights
  weights = metadata(1:nrpairs,6);
  % skip pairs with higher-dimensional cause/effect
  for pair=1:nrpairs
    if metadata(pair,2) ~= metadata(pair,3) || metadata(pair,4) ~= metadata(pair,5)
      weights(pair) = 0;
    end
  end

  % determine uniqueness of cause/effect values
  uniqueness = zeros(nrpairs,2);
  for pair=1:nrpairs
    % load data
    filename = sprintf('%s/pair%04d.txt',webdavdir,pair);
    data = load('-ascii',filename);
    % assign cause and effect
    cause = data(:,metadata(pair,2):metadata(pair,3));
    effect = data(:,metadata(pair,4):metadata(pair,5));
    uniq_cause = unique(cause,'rows');
    uniq_effect = unique(effect,'rows');
    uniqueness(pair,1) = size(uniq_cause,1) / size(cause,1);
    uniqueness(pair,2) = size(uniq_effect,1) / size(effect,1);
  end

  % ignore pairs that are too discrete
  for pair=1:nrpairs
    if uniqueness(pair,1) < uniquenessthreshold || uniqueness(pair,2) < uniquenessthreshold
      weights(pair) = 0;
    end
  end

  % read decisions
  % decisions{m,nrdec,pair} contains a list with length number of repetitions of the experiment
  % for each repetition, that slice of decisions{m,nrdec,pair} gives the
  % decision for the pair 'pair' and method 'm', where 'nrdec' gives the
  % number of decisions requested out of all pairs for method 'm'
  decisions = cell(nrmethods,nrpairs,nrpairs);
  for method=1:nrmethods
    methoddir = dir(sprintf('%s%s',prefix,methods{method}));
    names = {methoddir.name};
    isdir = [methoddir.isdir];
    for entry=1:length(names)
      if isdir(entry) && ~strcmp(names{entry},'.') && ~strcmp(names{entry},'..')
        % check whether the name is an integer
        if strcmp(int2str(str2double(names{entry})),names{entry})
          rep = str2double(names{entry});
          if rep <= maxrep
            fprintf('Reading method %s, repetition %d...\n', methods{method}, rep);
            alpha = zeros(nrpairs,1);
            dec = zeros(nrpairs,1);
            for pair=1:nrpairs
              filename = sprintf('%s%s/%d/pair%04d.hdf5',prefix,methods{method},rep,pair);
              if exist(filename,'file')
                fprintf('Reading %s\n',filename);
                result = load('-mat',filename);
  %              result = load('-hdf5',filename);  % FOR OCTAVE
fprintf('Pair %d , Forward conf: %e , Backward conf: %e\n', pair, result.forward.conf, result.backward.conf);
                if isfield(result,'conf')
                  dec(pair) = result.decision;
                  alpha(pair) = result.conf;
                if dec(pair) == 0 % if no decision, take one but with ultralow confidence
                        dec(pair) = 1;
                        alpha(pair) = -Inf;
                end
                else
                  dec(pair) = nan;
                  alpha(pair) = -Inf;
                end
              else
                fprintf('Skipping %s\n',filename);
                dec(pair) = nan;
                alpha(pair) = -Inf;
              end
              if isnan(alpha(pair)) || isnan(dec(pair))
                alpha(pair) = -Inf;
                dec(pair) = 0;
              end
            end
            [alphas,idx] = sort(alpha,1,'descend');
            rank = zeros(nrpairs,1);
            for i = 1:nrpairs
              rank(idx(i)) = i;
            end
            for i=1:nrpairs
              fprintf('Pair %d has rank %d (conf = %e)\n',idx(i),i,alpha(idx(i)));
            end
            for nrdec = 1:nrpairs
              for pair = 1:nrpairs
                decisions{method,nrdec,pair} = [decisions{method,nrdec,pair}; dec(pair) .* (rank(pair) <= nrdec)];
              end
            end
          end
        end
      end
    end
  end

return
